@@ -12,22 +12,20 @@
 
 #ifndef BIGRATIONAL_HEADER_
 #define BIGRATIONAL_HEADER_ 1
-#include <gmp.h>
-#include <R.h>
-#include <Rdefines.h>
 
 #include <string>
+
+#include "Rgmp.h"
 #include "biginteger.h"
 
-
 /**
- * \brief Class for rational value. 
+ * \brief Class for rational value.
  *
- * A big rational. Actually a wrapper for mpq_t to work with plus 
+ * A big rational. Actually a wrapper for mpq_t to work with plus
  * some special stuff.
  *
- * The bigrational special state "NA" means, no value is assigned. 
- * This does not mean, the internal state is not constructed, but 
+ * The bigrational special state "NA" means, no value is assigned.
+ * This does not mean, the internal state is not constructed, but
  * the value explicit is "not available".
  */
 class bigrational
@@ -38,35 +36,35 @@ class bigrational
    * The rational = n/d .
    */
   mpq_t value ;
-  
+
   /**
    * \brief a flag: true => NA value.
    */
   bool na;
 
- public:      
+ public:
   /**
    * Construct a "NA" biginteger.
    */
-  bigrational() : 
+  bigrational() :
     value(),
     na(true) {mpq_init(value);}
 
   /**
-   * Construct a biginteger from a raw expression. 
+   * Construct a biginteger from a raw expression.
    * in fact in raw: we will store an mpz_export of
    * either numeraotr, or denominator
    */
   bigrational(void* raw);
 
   /**
-   * Create a biginteger from a value. Remember to free the 
-   * parameter's mpz_t if you allocated them by yourself - 
+   * Create a biginteger from a value. Remember to free the
+   * parameter's mpz_t if you allocated them by yourself -
    * biginteger will copy the value.
    */
-  bigrational(const mpq_t& value_) : 
+  bigrational(const mpq_t& value_) :
     value(),
-    na(false) 
+    na(false)
     {
       mpq_init(value);
       mpq_set(value, value_);
@@ -75,49 +73,49 @@ class bigrational
   /**
    * \brief create a rational from an [big] integer
    */
-  bigrational(const mpz_t& value_) : 
+  bigrational(const mpz_t& value_) :
     value(),
-    na(false) 
+    na(false)
     {
       mpq_init(value);
       mpq_set_z(value, value_);
     }
-    
+
   /**
    * Construct a biginteger from a long value.
    */
-  bigrational(int value_) : 
+  bigrational(int value_) :
     value(),
     na(false) {
     mpq_init(value);
     if(value_ ==  NA_INTEGER)
       na = true  ;
-    else	
+    else
       mpq_set_si(value, value_,1);
   }
-    
+
   /**
    * Construct a biginteger from a long value.
    */
-  bigrational(int num_, int den_) : 
+  bigrational(int num_, int den_) :
     value(),
     na(false) {
     mpq_init(value);
     if((num_ ==  NA_INTEGER) || (den_ == NA_INTEGER) )
       na = true  ;
-    else	
+    else
       mpq_set_si(value, num_,den_);}
-    
+
   /**
    * Construct a biginteger from a double value.
    */
-  bigrational(double value_) : 
+  bigrational(double value_) :
     value(),
-    na(false) 
+    na(false)
     {
       mpq_init(value);
       if(R_FINITE( value_ ) )
-	mpq_set_d(value, value_);  
+	mpq_set_d(value, value_);
       else
 	na = true  ;
     }
@@ -125,9 +123,9 @@ class bigrational
   /**
    * Construct a biginteger from a string value. it can be "4343" or "2322/4343"
    */
-  bigrational(const std::string& value_) : 
+  bigrational(const std::string& value_) :
     value(),
-    na(false) 
+    na(false)
     {
       mpq_init(value);
       /* mpz_init.. return -1 when error, 0: ok */
@@ -136,13 +134,13 @@ class bigrational
       /*	if(mpz_init_set_str(value, value_.c_str(), 0) == -1)
 		Rf_error("Not a valid number");    */
     }
-    
+
   /**
    *  Copy constructor (mpz_t aren't standard-copyable)
    */
-  bigrational(const bigrational & rhs) : 
+  bigrational(const bigrational & rhs) :
     value(),
-    na(rhs.na) 
+    na(rhs.na)
     {
       mpq_init(value);
       mpq_set(value, rhs.value);
@@ -155,7 +153,7 @@ class bigrational
 
   /** \brief overload affectation operator
    *
-   */  
+   */
   bigrational & operator= (const bigrational& rhs);
 
   /**
@@ -173,25 +171,25 @@ class bigrational
   /** \brief Set Denominator: return value = value / value_ */
   void setDenValue(const mpq_t value_ ) {
     if(!na)
-      mpq_div(value,value, value_); 
+      mpq_div(value,value, value_);
   }
 
   /**
    * \brief set value from integer
    */
-  void setValue(int value_) {      
+  void setValue(int value_) {
     if(value_ == NA_INTEGER)
       {mpq_set_ui(value, 0,1); na = true  ;}
     else
       {
-	mpq_set_si(value, value_,1); 
+	mpq_set_si(value, value_,1);
 	na = false;
       }
   }
-    
+
   /** \brief set value from unsigned int
    */
-  void setValue(unsigned long int value_) {      
+  void setValue(unsigned long int value_) {
     if((int)value_ == NA_INTEGER)
       {mpq_set_ui(value, 0,1); na = true  ;}
     else
@@ -200,7 +198,7 @@ class bigrational
 
   /** \brief set value from float (double)
    */
-  void setValue(double value_) {      
+  void setValue(double value_) {
     if( R_FINITE (value_ ) )
       {mpq_set_d(value, value_); na = false;}
     else
@@ -209,7 +207,7 @@ class bigrational
 
 
   /**
-   * \brief set value from big integer 
+   * \brief set value from big integer
    */
   void setValue(const mpz_t & value_) {
     mpq_set_z(value,value_);
@@ -224,7 +222,7 @@ class bigrational
       mpq_set_z(value,value_.getValueTemp());
       na = value_.isNA();
     }
-    
+
   /**
    * \brief set value from biginteger value
    */
@@ -233,7 +231,7 @@ class bigrational
       mpq_set(value,value_.getValueTemp());
       na = value_.isNA();
     }
-    
+
   /**
    * For const-purposes, return the value. Remember, that the return value
    * only lives as long as this class live, so do not call getValueTemp on
@@ -245,7 +243,7 @@ class bigrational
    * Return true, if the value is NA.
    */
   bool isNA() const {return na;}
-    
+
   /**
    * set NA value
    */
@@ -255,14 +253,14 @@ class bigrational
    * Return 1, if the value is > 0;  -1 when negative, 0 when 0.
    */
   int sgn() const {return mpq_sgn(value);}
-    
+
   /**
    *  Convert the biginteger into a standard string.
    */
   std::string str(int b) const;
 
   /**
-   * \brief Convert the biginteger into a double value 
+   * \brief Convert the biginteger into a double value
    * (you may loose precision)
    */
   double as_double() const {return mpq_get_d(value);}
@@ -274,7 +272,7 @@ class bigrational
    *
    * Also remember, that the modulus is not saved this way. To obtain a
    * modulus raw byte use get_modulus().as_raw(void*).
-   * 
+   *
    * @return number of bytes used (same as raw_size())
    */
   //  int as_raw(void* raw) const;
@@ -295,13 +293,13 @@ class bigrational
    */
   bigrational inv();
 
-  
+
 };
 
 
 /** \brief mpq struct.
  *
- * Use this to clear mpq_t structs at end-of-function automatically 
+ * Use this to clear mpq_t structs at end-of-function automatically
  */
 struct mpq_t_sentry {
   /** the bigrational */
@@ -309,7 +307,7 @@ struct mpq_t_sentry {
   /** initialisation */
   mpq_t_sentry(mpq_t& v): value(v) {}
   /** free all */
-  ~mpq_t_sentry() 
+  ~mpq_t_sentry()
   {
     mpq_clear(value);
   }
